@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
+// CA 94945, U.S.A., +1(415)492-9861, for further information.
+
 #ifndef MUPDF_FITZ_FONT_H
 #define MUPDF_FITZ_FONT_H
 
@@ -5,23 +27,54 @@
 #include "mupdf/fitz/context.h"
 #include "mupdf/fitz/geometry.h"
 #include "mupdf/fitz/buffer.h"
+#include "mupdf/fitz/color.h"
 
 /* forward declaration for circular dependency */
-struct fz_device_s;
+struct fz_device;
 
-/*
+/* Various font encoding tables and lookup functions */
+
+FZ_DATA extern const char *fz_glyph_name_from_adobe_standard[256];
+FZ_DATA extern const char *fz_glyph_name_from_iso8859_7[256];
+FZ_DATA extern const char *fz_glyph_name_from_koi8u[256];
+FZ_DATA extern const char *fz_glyph_name_from_mac_expert[256];
+FZ_DATA extern const char *fz_glyph_name_from_mac_roman[256];
+FZ_DATA extern const char *fz_glyph_name_from_win_ansi[256];
+FZ_DATA extern const char *fz_glyph_name_from_windows_1252[256];
+
+FZ_DATA extern const unsigned short fz_unicode_from_iso8859_1[256];
+FZ_DATA extern const unsigned short fz_unicode_from_iso8859_7[256];
+FZ_DATA extern const unsigned short fz_unicode_from_koi8u[256];
+FZ_DATA extern const unsigned short fz_unicode_from_pdf_doc_encoding[256];
+FZ_DATA extern const unsigned short fz_unicode_from_windows_1250[256];
+FZ_DATA extern const unsigned short fz_unicode_from_windows_1251[256];
+FZ_DATA extern const unsigned short fz_unicode_from_windows_1252[256];
+
+int fz_iso8859_1_from_unicode(int u);
+int fz_iso8859_7_from_unicode(int u);
+int fz_koi8u_from_unicode(int u);
+int fz_windows_1250_from_unicode(int u);
+int fz_windows_1251_from_unicode(int u);
+int fz_windows_1252_from_unicode(int u);
+
+int fz_unicode_from_glyph_name(const char *name);
+int fz_unicode_from_glyph_name_strict(const char *name);
+const char **fz_duplicate_glyph_names_from_unicode(int unicode);
+const char *fz_glyph_name_from_unicode_sc(int unicode);
+
+/**
 	An abstract font handle.
 */
-typedef struct fz_font_s fz_font;
+typedef struct fz_font fz_font;
 
-/*
- * Fonts come in two variants:
- *	Regular fonts are handled by FreeType.
- *	Type 3 fonts have callbacks to the interpreter.
- */
+/**
+	Fonts come in two variants:
+	Regular fonts are handled by FreeType.
+	Type 3 fonts have callbacks to the interpreter.
+*/
 
-/*
-	fz_font_ft_face: Retrieve the FT_Face handle
+/**
+	Retrieve the FT_Face handle
 	for the font.
 
 	font: The font to query
@@ -32,8 +85,8 @@ typedef struct fz_font_s fz_font;
 */
 void *fz_font_ft_face(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_t3_procs: Retrieve the Type3 procs
+/**
+	Retrieve the Type3 procs
 	for a font.
 
 	font: The font to query
@@ -43,22 +96,11 @@ void *fz_font_ft_face(fz_context *ctx, fz_font *font);
 */
 fz_buffer **fz_font_t3_procs(fz_context *ctx, fz_font *font);
 
-/*
-	ft_error_string: map an FT error number to a
-	static string.
-
-	err: The error number to lookup.
-
-	Returns a pointer to a static textual representation
-	of a freetype error.
-*/
-const char *ft_error_string(int err);
-
 /* common CJK font collections */
 enum { FZ_ADOBE_CNS, FZ_ADOBE_GB, FZ_ADOBE_JAPAN, FZ_ADOBE_KOREA };
 
-/*
-	fz_font_flags_t: Every fz_font carries a set of flags
+/**
+	Every fz_font carries a set of flags
 	within it, in a fz_font_flags_t structure.
 */
 typedef struct
@@ -74,10 +116,13 @@ typedef struct
 	unsigned int fake_italic : 1; /* synthesize italic */
 	unsigned int has_opentype : 1; /* has opentype shaping tables */
 	unsigned int invalid_bbox : 1;
+
+	unsigned int cjk : 1;
+	unsigned int cjk_lang : 2; /* CNS, GB, JAPAN, or KOREA */
 } fz_font_flags_t;
 
-/*
-	fz_font_flags: Retrieve a pointer to the font flags
+/**
+	Retrieve a pointer to the font flags
 	for a given font. These can then be updated as required.
 
 	font: The font to query
@@ -87,8 +132,8 @@ typedef struct
 */
 fz_font_flags_t *fz_font_flags(fz_font *font);
 
-/*
-	fz_shaper_data_t: In order to shape a given font, we need to
+/**
+	In order to shape a given font, we need to
 	declare it to a shaper library (harfbuzz, by default, but others
 	are possible). To avoid redeclaring it every time we need to
 	shape, we hold a shaper handle and the destructor for it within
@@ -102,8 +147,8 @@ typedef struct
 	void (*destroy)(fz_context *ctx, void *); /* Destructor for shape_handle */
 } fz_shaper_data_t;
 
-/*
-	fz_shaper_data_t: Retrieve a pointer to the shaper data
+/**
+	Retrieve a pointer to the shaper data
 	structure for the given font.
 
 	font: The font to query.
@@ -113,8 +158,8 @@ typedef struct
 */
 fz_shaper_data_t *fz_font_shaper_data(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_name: Retrieve a pointer to the name of the font.
+/**
+	Retrieve a pointer to the name of the font.
 
 	font: The font to query.
 
@@ -123,74 +168,83 @@ fz_shaper_data_t *fz_font_shaper_data(fz_context *ctx, fz_font *font);
 */
 const char *fz_font_name(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_is_bold: Returns true if the font is bold.
+/**
+	Query whether the font flags say that this font is bold.
 */
 int fz_font_is_bold(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_is_italic: Returns true if the font is italic.
+/**
+	Query whether the font flags say that this font is italic.
 */
 int fz_font_is_italic(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_is_serif: Returns true if the font is serif.
+/**
+	Query whether the font flags say that this font is serif.
 */
 int fz_font_is_serif(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_is_monospaced: Returns true if the font is monospaced.
+/**
+	Query whether the font flags say that this font is monospaced.
 */
 int fz_font_is_monospaced(fz_context *ctx, fz_font *font);
 
-/*
-	fz_font_bbox: Retrieve a pointer to the font bbox.
+/**
+	Retrieve the font bbox.
 
 	font: The font to query.
 
-	Returns a pointer to the font bbox (or NULL if the
-	font is NULL).
+	Returns the font bbox by value; it is valid only if
+	fz_font_flags(font)->invalid_bbox is zero.
 */
 fz_rect fz_font_bbox(fz_context *ctx, fz_font *font);
 
-/*
-	fz_load_system_font_fn: Type for user supplied system font loading hook.
+/**
+	Type for user supplied system font loading hook.
 
 	name: The name of the font to load.
+
 	bold: 1 if a bold font desired, 0 otherwise.
+
 	italic: 1 if an italic font desired, 0 otherwise.
-	needs_exact_metrics: 1 if an exact metric match is required for the font requested.
+	needs_exact_metrics: 1 if an exact metric match is required for
+	the font requested.
 
 	Returns a new font handle, or NULL if no font found (or on error).
 */
 typedef fz_font *(fz_load_system_font_fn)(fz_context *ctx, const char *name, int bold, int italic, int needs_exact_metrics);
 
-/*
-	fz_load_system_cjk_font_fn: Type for user supplied cjk font loading hook.
+/**
+	Type for user supplied cjk font loading hook.
 
 	name: The name of the font to load.
-	ordering: The ordering for which to load the font (e.g. FZ_ADOBE_KOREA)
+
+	ordering: The ordering for which to load the font (e.g.
+	FZ_ADOBE_KOREA)
+
 	serif: 1 if a serif font is desired, 0 otherwise.
 
 	Returns a new font handle, or NULL if no font found (or on error).
 */
 typedef fz_font *(fz_load_system_cjk_font_fn)(fz_context *ctx, const char *name, int ordering, int serif);
 
-/*
-	fz_load_system_fallback_font_fn: Type for user supplied fallback font loading hook.
+/**
+	Type for user supplied fallback font loading hook.
 
 	name: The name of the font to load.
+
 	script: UCDN script enum.
+
 	language: FZ_LANG enum.
+
 	serif, bold, italic: boolean style flags.
 
 	Returns a new font handle, or NULL if no font found (or on error).
 */
 typedef fz_font *(fz_load_system_fallback_font_fn)(fz_context *ctx, int script, int language, int serif, int bold, int italic);
 
-/*
-	fz_install_load_system_font_fn: Install functions to allow
-	MuPDF to request fonts from the system.
+/**
+	Install functions to allow MuPDF to request fonts from the
+	system.
 
 	Only one set of hooks can be in use at a time.
 */
@@ -199,10 +253,8 @@ void fz_install_load_system_font_funcs(fz_context *ctx,
 	fz_load_system_cjk_font_fn *f_cjk,
 	fz_load_system_fallback_font_fn *f_fallback);
 
-/* fz_load_*_font returns NULL if no font could be loaded (also on error) */
-/*
-	fz_load_system_font: Attempt to load a given font from the
-	system.
+/**
+	Attempt to load a given font from the system.
 
 	name: The name of the desired font.
 
@@ -218,8 +270,8 @@ void fz_install_load_system_font_funcs(fz_context *ctx,
 */
 fz_font *fz_load_system_font(fz_context *ctx, const char *name, int bold, int italic, int needs_exact_metrics);
 
-/*
-	fz_load_system_cjk_font: Attempt to load a given font from
+/**
+	Attempt to load a given font from
 	the system.
 
 	name: The name of the desired font.
@@ -233,8 +285,8 @@ fz_font *fz_load_system_font(fz_context *ctx, const char *name, int bold, int it
 */
 fz_font *fz_load_system_cjk_font(fz_context *ctx, const char *name, int ordering, int serif);
 
-/*
-	fz_lookup_builtin_font: Search the builtin fonts for a match.
+/**
+	Search the builtin fonts for a match.
 	Whether a given font is present or not will depend on the
 	configuration in which MuPDF is built.
 
@@ -251,8 +303,8 @@ fz_font *fz_load_system_cjk_font(fz_context *ctx, const char *name, int ordering
 */
 const unsigned char *fz_lookup_builtin_font(fz_context *ctx, const char *name, int bold, int italic, int *len);
 
-/*
-	fz_lookup_base14_font: Search the builtin base14 fonts for a match.
+/**
+	Search the builtin base14 fonts for a match.
 	Whether a given font is present or not will depend on the
 	configuration in which MuPDF is built.
 
@@ -265,41 +317,46 @@ const unsigned char *fz_lookup_builtin_font(fz_context *ctx, const char *name, i
 */
 const unsigned char *fz_lookup_base14_font(fz_context *ctx, const char *name, int *len);
 
-/*
-	fz_lookup_icc: Search for icc profile.
-
-	name: The name of the profile desired (gray-icc, rgb-icc, cmyk-icc or lab-icc).
-
-	len: Pointer to a place to receive the length of the discovered.
-
-	Returns a pointer to the icc file data, or NULL if not present.
-*/
-const unsigned char *fz_lookup_icc(fz_context *ctx, enum fz_colorspace_type name, size_t *len);
-
-/*
-	fz_lookup_cjk_font: Search the builtin cjk fonts for a match.
+/**
+	Search the builtin cjk fonts for a match.
 	Whether a font is present or not will depend on the
 	configuration in which MuPDF is built.
 
 	ordering: The desired ordering of the font (e.g. FZ_ADOBE_KOREA).
 
-	size: Pointer to a place to receive the length of the discovered font buffer.
-
-	subfont: Pointer to a place to store the subfont index of the discovered font.
+	len: Pointer to a place to receive the length of the discovered
+	font buffer.
 
 	Returns a pointer to the font file data, or NULL if not present.
 */
 const unsigned char *fz_lookup_cjk_font(fz_context *ctx, int ordering, int *len, int *index);
-const unsigned char *fz_lookup_cjk_font_by_language(fz_context *ctx, const char *lang, int *size, int *subfont);
 
-/*
-	fz_lookup_cjk_ordering_by_language: Return the matching FZ_ADOBE_* ordering
+/**
+	Search the builtin cjk fonts for a match for a given language.
+	Whether a font is present or not will depend on the
+	configuration in which MuPDF is built.
+
+	lang: Pointer to a (case sensitive) language string (e.g.
+	"ja", "ko", "zh-Hant" etc).
+
+	len: Pointer to a place to receive the length of the discovered
+	font buffer.
+
+	subfont: Pointer to a place to store the subfont index of the
+	discovered font.
+
+	Returns a pointer to the font file data, or NULL if not present.
+*/
+const unsigned char *fz_lookup_cjk_font_by_language(fz_context *ctx, const char *lang, int *len, int *subfont);
+
+/**
+	Return the matching FZ_ADOBE_* ordering
 	for the given language tag, such as "zh-Hant", "zh-Hans", "ja", or "ko".
 */
 int fz_lookup_cjk_ordering_by_language(const char *name);
 
-/*
-	fz_lookup_noto_font: Search the builtin noto fonts for a match.
+/**
+	Search the builtin noto fonts for a match.
 	Whether a font is present or not will depend on the
 	configuration in which MuPDF is built.
 
@@ -314,33 +371,19 @@ int fz_lookup_cjk_ordering_by_language(const char *name);
 */
 const unsigned char *fz_lookup_noto_font(fz_context *ctx, int script, int lang, int *len, int *subfont);
 
-/*
-	fz_lookup_noto_symbol[12]_font: Search the builtin noto fonts
-	for a symbol font. Whether a font is present or not will
-	depend on the configuration in which MuPDF is built.
-
-	len: Pointer to a place to receive the length of the discovered
-	font buffer.
-
-	Returns a pointer to the font file data, or NULL if not present.
+/**
+	Search the builtin noto fonts specific symbol fonts.
+	Whether a font is present or not will depend on the
+	configuration in which MuPDF is built.
 */
+const unsigned char *fz_lookup_noto_math_font(fz_context *ctx, int *len);
+const unsigned char *fz_lookup_noto_music_font(fz_context *ctx, int *len);
 const unsigned char *fz_lookup_noto_symbol1_font(fz_context *ctx, int *len);
 const unsigned char *fz_lookup_noto_symbol2_font(fz_context *ctx, int *len);
-
-/*
-	fz_lookup_noto_emoji_font: Search the builtin noto fonts
-	for an emoji font. Whether a font is present or not will
-	depend on the configuration in which MuPDF is built.
-
-	len: Pointer to a place to receive the length of the discovered
-	font buffer.
-
-	Returns a pointer to the font file data, or NULL if not present.
-*/
 const unsigned char *fz_lookup_noto_emoji_font(fz_context *ctx, int *len);
 
-/*
-	fz_load_fallback_font: Try to load a fallback font for the
+/**
+	Try to load a fallback font for the
 	given combination of font attributes. Whether a font is
 	present or not will depend on the configuration in which
 	MuPDF is built.
@@ -359,8 +402,8 @@ const unsigned char *fz_lookup_noto_emoji_font(fz_context *ctx, int *len);
 */
 fz_font *fz_load_fallback_font(fz_context *ctx, int script, int language, int serif, int bold, int italic);
 
-/*
-	fz_new_type3_font: Create a new (empty) type3 font.
+/**
+	Create a new (empty) type3 font.
 
 	name: Name of font (or NULL).
 
@@ -371,8 +414,8 @@ fz_font *fz_load_fallback_font(fz_context *ctx, int script, int language, int se
 */
 fz_font *fz_new_type3_font(fz_context *ctx, const char *name, fz_matrix matrix);
 
-/*
-	fz_new_font_from_memory: Create a new font from a font
+/**
+	Create a new font from a font
 	file in memory.
 
 	name: Name of font (leave NULL to use name from font).
@@ -389,9 +432,8 @@ fz_font *fz_new_type3_font(fz_context *ctx, const char *name, fz_matrix matrix);
 */
 fz_font *fz_new_font_from_memory(fz_context *ctx, const char *name, const unsigned char *data, int len, int index, int use_glyph_bbox);
 
-/*
-	fz_new_font_from_buffer: Create a new font from a font
-	file in a fz_buffer.
+/**
+	Create a new font from a font file in a fz_buffer.
 
 	name: Name of font (leave NULL to use name from font).
 
@@ -405,9 +447,8 @@ fz_font *fz_new_font_from_memory(fz_context *ctx, const char *name, const unsign
 */
 fz_font *fz_new_font_from_buffer(fz_context *ctx, const char *name, fz_buffer *buffer, int index, int use_glyph_bbox);
 
-/*
-	fz_new_font_from_file: Create a new font from a font
-	file.
+/**
+	Create a new font from a font file.
 
 	name: Name of font (leave NULL to use name from font).
 
@@ -421,12 +462,14 @@ fz_font *fz_new_font_from_buffer(fz_context *ctx, const char *name, fz_buffer *b
 */
 fz_font *fz_new_font_from_file(fz_context *ctx, const char *name, const char *path, int index, int use_glyph_bbox);
 
-/* Create a new font from one of the built-in fonts. */
+/**
+	Create a new font from one of the built-in fonts.
+*/
 fz_font *fz_new_base14_font(fz_context *ctx, const char *name);
 fz_font *fz_new_cjk_font(fz_context *ctx, int ordering);
 fz_font *fz_new_builtin_font(fz_context *ctx, const char *name, int is_bold, int is_italic);
 
-/*
+/**
 	Add a reference to an existing fz_font.
 
 	font: The font to add a reference to.
@@ -435,7 +478,7 @@ fz_font *fz_new_builtin_font(fz_context *ctx, const char *name, int is_bold, int
 */
 fz_font *fz_keep_font(fz_context *ctx, fz_font *font);
 
-/*
+/**
 	Drop a reference to a fz_font, destroying the
 	font when the last reference is dropped.
 
@@ -443,8 +486,8 @@ fz_font *fz_keep_font(fz_context *ctx, fz_font *font);
 */
 void fz_drop_font(fz_context *ctx, fz_font *font);
 
-/*
-	fz_set_font_bbox: Set the font bbox.
+/**
+	Set the font bbox.
 
 	font: The font to set the bbox for.
 
@@ -452,8 +495,8 @@ void fz_drop_font(fz_context *ctx, fz_font *font);
 */
 void fz_set_font_bbox(fz_context *ctx, fz_font *font, float xmin, float ymin, float xmax, float ymax);
 
-/*
-	fz_bound_glyph: Return a bbox for a given glyph in a font.
+/**
+	Return a bbox for a given glyph in a font.
 
 	font: The font to look for the glyph in.
 
@@ -461,14 +504,13 @@ void fz_set_font_bbox(fz_context *ctx, fz_font *font, float xmin, float ymin, fl
 
 	trm: The matrix to apply to the glyph before bounding.
 
-	r: Pointer to a fz_rect to use for storage.
-
-	Returns r, after filling it in with the bounds of the given glyph.
+	Returns rectangle by value containing the bounds of the given
+	glyph.
 */
 fz_rect fz_bound_glyph(fz_context *ctx, fz_font *font, int gid, fz_matrix trm);
 
-/*
-	fz_glyph_cacheable: Determine if a given glyph in a font
+/**
+	Determine if a given glyph in a font
 	is cacheable. Certain glyphs in a type 3 font cannot safely
 	be cached, as their appearance depends on the enclosing
 	graphic state.
@@ -481,8 +523,8 @@ fz_rect fz_bound_glyph(fz_context *ctx, fz_font *font, int gid, fz_matrix trm);
 */
 int fz_glyph_cacheable(fz_context *ctx, fz_font *font, int gid);
 
-/*
-	fz_run_t3_glyph: Run a glyph from a Type3 font to
+/**
+	Run a glyph from a Type3 font to
 	a given device.
 
 	font: The font to find the glyph in.
@@ -493,28 +535,10 @@ int fz_glyph_cacheable(fz_context *ctx, fz_font *font, int gid);
 
 	dev: The device to render onto.
 */
-void fz_run_t3_glyph(fz_context *ctx, fz_font *font, int gid, fz_matrix trm, struct fz_device_s *dev);
+void fz_run_t3_glyph(fz_context *ctx, fz_font *font, int gid, fz_matrix trm, struct fz_device *dev);
 
-/*
-	fz_decouple_type3_font: Internal function to remove the
-	references to a document held by a Type3 font. This is
-	called during document destruction to ensure that Type3
-	fonts clean up properly.
-
-	Without this call being made, Type3 fonts can be left
-	holding pdf_obj references for the sake of interpretation
-	operations that will never come. These references
-	cannot be freed after the document, hence this function
-	forces them to be freed earlier in the process.
-
-	font: The font to decouple.
-
-	t3doc: The document to which the font may refer.
-*/
-void fz_decouple_type3_font(fz_context *ctx, fz_font *font, void *t3doc);
-
-/*
-	fz_advance_glyph: Return the advance for a given glyph.
+/**
+	Return the advance for a given glyph.
 
 	font: The font to look for the glyph in.
 
@@ -526,8 +550,8 @@ void fz_decouple_type3_font(fz_context *ctx, fz_font *font, void *t3doc);
 */
 float fz_advance_glyph(fz_context *ctx, fz_font *font, int glyph, int wmode);
 
-/*
-	fz_encode_character: Find the glyph id for a given unicode
+/**
+	Find the glyph id for a given unicode
 	character within a font.
 
 	font: The font to look for the unicode character in.
@@ -539,8 +563,31 @@ float fz_advance_glyph(fz_context *ctx, fz_font *font, int glyph, int wmode);
 */
 int fz_encode_character(fz_context *ctx, fz_font *font, int unicode);
 
-/*
-	fz_encode_character_with_fallback: Find the glyph id for
+/**
+	Encode character, preferring small-caps variant if available.
+
+	font: The font to look for the unicode character in.
+
+	unicode: The unicode character to encode.
+
+	Returns the glyph id for the given unicode value, or 0 if
+	unknown.
+*/
+int fz_encode_character_sc(fz_context *ctx, fz_font *font, int unicode);
+
+/**
+	Encode character.
+
+	Either by direct lookup of glyphname within a font, or, failing
+	that, by mapping glyphname to unicode and thence to the glyph
+	index within the given font.
+
+	Returns zero for type3 fonts.
+*/
+int fz_encode_character_by_glyph_name(fz_context *ctx, fz_font *font, const char *glyphname);
+
+/**
+	Find the glyph id for
 	a given unicode character within a font, falling back to
 	an alternative if not found.
 
@@ -566,8 +613,8 @@ int fz_encode_character(fz_context *ctx, fz_font *font, int unicode);
 */
 int fz_encode_character_with_fallback(fz_context *ctx, fz_font *font, int unicode, int script, int language, fz_font **out_font);
 
-/*
-	fz_get_glyph_name: Find the name of a glyph
+/**
+	Find the name of a glyph
 
 	font: The font to look for the glyph in.
 
@@ -584,30 +631,99 @@ int fz_encode_character_with_fallback(fz_context *ctx, fz_font *font, int unicod
 */
 void fz_get_glyph_name(fz_context *ctx, fz_font *font, int glyph, char *buf, int size);
 
-/*
-	Get font ascender and descender values.
+/**
+	Retrieve font ascender in ems.
 */
 float fz_font_ascender(fz_context *ctx, fz_font *font);
+
+/**
+	Retrieve font descender in ems.
+*/
 float fz_font_descender(fz_context *ctx, fz_font *font);
 
+/**
+	Retrieve the MD5 digest for the font's data.
+*/
 void fz_font_digest(fz_context *ctx, fz_font *font, unsigned char digest[16]);
 
-/*
+/* Implementation details: subject to change. */
+
+void fz_decouple_type3_font(fz_context *ctx, fz_font *font, void *t3doc);
+
+/**
+	map an FT error number to a
+	static string.
+
+	err: The error number to lookup.
+
+	Returns a pointer to a static textual representation
+	of a freetype error.
+*/
+const char *ft_error_string(int err);
+int ft_char_index(void *face, int cid);
+int ft_name_index(void *face, const char *name);
+
+/**
 	Internal functions for our Harfbuzz integration
 	to work around the lack of thread safety.
 */
 
-/*
-	fz_hb_lock: Lock against Harfbuzz being called
+/**
+	Lock against Harfbuzz being called
 	simultaneously in several threads. This reuses
 	FZ_LOCK_FREETYPE.
 */
 void fz_hb_lock(fz_context *ctx);
 
-/*
-	fz_hb_unlock: Unlock after a Harfbuzz call. This reuses
+/**
+	Unlock after a Harfbuzz call. This reuses
 	FZ_LOCK_FREETYPE.
 */
 void fz_hb_unlock(fz_context *ctx);
 
+struct fz_font
+{
+	int refs;
+	char name[32];
+	fz_buffer *buffer;
+
+	fz_font_flags_t flags;
+
+	void *ft_face; /* has an FT_Face if used */
+	fz_shaper_data_t shaper_data;
+
+	fz_matrix t3matrix;
+	void *t3resources;
+	fz_buffer **t3procs; /* has 256 entries if used */
+	struct fz_display_list **t3lists; /* has 256 entries if used */
+	float *t3widths; /* has 256 entries if used */
+	unsigned short *t3flags; /* has 256 entries if used */
+	void *t3doc; /* a pdf_document for the callback */
+	void (*t3run)(fz_context *ctx, void *doc, void *resources, fz_buffer *contents, struct fz_device *dev, fz_matrix ctm, void *gstate, fz_default_colorspaces *default_cs);
+	void (*t3freeres)(fz_context *ctx, void *doc, void *resources);
+
+	fz_rect bbox;	/* font bbox is used only for t3 fonts */
+
+	int glyph_count;
+
+	/* per glyph bounding box cache. */
+	fz_rect **bbox_table;
+	int use_glyph_bbox;
+
+	/* substitute metrics */
+	int width_count;
+	short width_default; /* in 1000 units */
+	short *width_table; /* in 1000 units */
+
+	/* cached glyph metrics */
+	float **advance_cache;
+
+	/* cached encoding lookup */
+	uint16_t *encoding_cache[256];
+
+	/* cached md5sum for caching */
+	int has_digest;
+	unsigned char digest[16];
+};
+
 #endif
